@@ -200,7 +200,7 @@ class VirtualFrame : public ZoneObject {
   inline void PrepareForReturn();
 
   // Number of local variables after when we use a loop for allocating.
-  static const int kLocalVarBound = 7;
+  static const int kLocalVarBound = 14;
 
   // Allocate and initialize the frame-allocated locals.
   void AllocateStackSlots();
@@ -341,7 +341,7 @@ class VirtualFrame : public ZoneObject {
   // and by the order of the three arguments on the frame.
   Result CallCommonStoreIC(Handle<Code> ic,
                            Result* value,
-                           Result *key,
+                           Result* key,
                            Result* receiver);
 
   // Call store IC.  Name, value, and receiver are found on top
@@ -354,6 +354,10 @@ class VirtualFrame : public ZoneObject {
     return CallCommonStoreIC(ic, &value, &name, &receiver);
   }
 
+  // Call store IC.  If the load is contextual, value is found on top of the
+  // frame.  If not, value and receiver are on the frame.  Both are dropped.
+  Result CallStoreIC(Handle<String> name, bool is_contextual);
+
   // Call keyed store IC.  Value, key, and receiver are found on top
   // of the frame.  All are dropped.
   Result CallKeyedStoreIC() {
@@ -369,6 +373,8 @@ class VirtualFrame : public ZoneObject {
   // The argument count does not include the receiver.
   Result CallCallIC(RelocInfo::Mode mode, int arg_count, int loop_nesting);
 
+  Result CallKeyedCallIC(RelocInfo::Mode mode, int arg_count, int loop_nesting);
+
   // Allocate and call JS function as constructor.  Arguments,
   // receiver (global object), and function are found on top of the
   // frame.  Function is not dropped.  The argument count does not
@@ -386,6 +392,13 @@ class VirtualFrame : public ZoneObject {
   // Duplicate the top element of the frame.
   void Dup() { PushFrameSlotAt(element_count() - 1); }
 
+  // Duplicate the n'th element from the top of the frame.
+  // Dup(1) is equivalent to Dup().
+  void Dup(int n) {
+    ASSERT(n > 0);
+    PushFrameSlotAt(element_count() - n);
+  }
+
   // Pop an element from the top of the expression stack.  Returns a
   // Result, which may be a constant or a register.
   Result Pop();
