@@ -11,7 +11,7 @@ var kPoolSize = 40*1024;
 fs.Stats = binding.Stats;
 
 fs.Stats.prototype._checkModeProperty = function (property) {
-  return ((this.mode & property) === property);
+  return ((this.mode & process.S_IFMT) === property);
 };
 
 fs.Stats.prototype.isDirectory = function () {
@@ -150,7 +150,7 @@ fs.openSync = function (path, flags, mode) {
 };
 
 fs.read = function (fd, buffer, offset, length, position, callback) {
-  if (!(buffer instanceof Buffer)) {
+  if (!Buffer.isBuffer(buffer)) {
     // legacy string interface (fd, length, position, encoding, callback)
     var cb = arguments[4],
         encoding = arguments[3];
@@ -175,7 +175,7 @@ fs.read = function (fd, buffer, offset, length, position, callback) {
 
 fs.readSync = function (fd, buffer, offset, length, position) {
   var legacy = false;
-  if (!(buffer instanceof Buffer)) {
+  if (!Buffer.isBuffer(buffer)) {
     // legacy string interface (fd, length, position, encoding, callback)
     legacy = true;
     encoding = arguments[3];
@@ -198,7 +198,7 @@ fs.readSync = function (fd, buffer, offset, length, position) {
 };
 
 fs.write = function (fd, buffer, offset, length, position, callback) {
-  if (!(buffer instanceof Buffer)) {
+  if (!Buffer.isBuffer(buffer)) {
     // legacy string interface (fd, data, position, encoding, callback)
     callback = arguments[4];
     position = arguments[2];
@@ -212,7 +212,7 @@ fs.write = function (fd, buffer, offset, length, position, callback) {
 };
 
 fs.writeSync = function (fd, buffer, offset, length, position) {
-  if (!(buffer instanceof Buffer)) {
+  if (!Buffer.isBuffer(buffer)) {
     // legacy string interface (fd, data, position, encoding)
     position = arguments[2];
 
@@ -221,7 +221,7 @@ fs.writeSync = function (fd, buffer, offset, length, position) {
     length = buffer.length;
   }
 
-  binding.write(fd, buffer, offset, length, position);
+  return binding.write(fd, buffer, offset, length, position);
 };
 
 fs.rename = function (oldPath, newPath, callback) {
@@ -360,17 +360,17 @@ fs.chownSync = function(path, uid, gid) {
   return binding.chown(path, uid, gid);
 };
 
-function writeAll (fd, data, encoding, callback) {
-  fs.write(fd, data, 0, encoding, function (writeErr, written) {
+function writeAll (fd, buffer, callback) {
+  fs.write(fd, buffer, 0, buffer.length, null, function (writeErr, written) {
     if (writeErr) {
       fs.close(fd, function () {
         if (callback) callback(writeErr);
       });
     } else {
-      if (written === data.length) {
+      if (written === buffer.length) {
         fs.close(fd, callback);
       } else {
-        writeAll(fd, data.slice(written), encoding, callback);
+        writeAll(fd, buffer.slice(written), callback);
       }
     }
   });
@@ -384,7 +384,8 @@ fs.writeFile = function (path, data, encoding_, callback) {
     if (openErr) {
       if (callback) callback(openErr);
     } else {
-      writeAll(fd, data, encoding, callback);
+      var buffer = Buffer.isBuffer(data) ? data : new Buffer(data, encoding);
+      writeAll(fd, buffer, callback);
     }
   });
 };
@@ -614,6 +615,16 @@ var ReadStream = fs.ReadStream = function(path, options) {
     this[key] = options[key];
   }
 
+  if(this.start || this.end) {
+    if(this.start === undefined || this.end === undefined) {
+      self.emit('error', new Error('Both start and end are needed for range streaming.'));
+    } else if(this.start > this.end) {
+      self.emit('error', new Error('start must be <= end'));
+    } else {
+      this.firstRead = true;
+    }
+  }
+
   if (this.fd !== null) {
     return;
   }
@@ -636,17 +647,8 @@ sys.inherits(ReadStream, events.EventEmitter);
 fs.FileReadStream = fs.ReadStream; // support the legacy name
 
 ReadStream.prototype.setEncoding = function (encoding) {
-  var Utf8Decoder = require("utf8decoder").Utf8Decoder; // lazy load
-  var self = this;
-  this._encoding = encoding.toLowerCase();
-  if (this._encoding == 'utf-8' || this._encoding == 'utf8') {
-    this._decoder = new Utf8Decoder();
-    this._decoder.onString = function(str) {
-      self.emit('data', str);
-    };
-  } else if (this._decoder) {
-    delete this._decoder;
-  }
+  var StringDecoder = require("string_decoder").StringDecoder; // lazy load
+  this._decoder = new StringDecoder(encoding);
 };
 
 
@@ -661,6 +663,11 @@ ReadStream.prototype._read = function () {
     allocNewPool();
   }
 
+  if(this.start && this.firstRead) {
+    this.pos = this.start;
+    this.firstRead = false;
+  }
+
   // Grab another reference to the pool in the case that while we're in the
   // thread pool another read() finishes up the pool, and allocates a new
   // one.
@@ -668,6 +675,10 @@ ReadStream.prototype._read = function () {
   var toRead = Math.min(pool.length - pool.used, this.bufferSize);
   var start = pool.used;
 
+  if(this.pos) {
+    toRead = Math.min(this.end - this.pos + 1, toRead);
+  }
+
   function afterRead (err, bytesRead) {
     if (err) {
       self.emit('error', err);
@@ -700,19 +711,21 @@ ReadStream.prototype._read = function () {
     self._read();
   }
 
-  fs.read(self.fd, pool, pool.used, toRead, undefined, afterRead);
+  fs.read(self.fd, pool, pool.used, toRead, this.pos, afterRead);
+
+  if(self.pos) {
+    self.pos += toRead;
+  }
   pool.used += toRead;
 };
 
 
 ReadStream.prototype._emitData = function (d) {
-  if (!this._encoding) {
-    this.emit('data', d);
-  } else if (this._decoder) {
-    this._decoder.write(d);
+  if (this._decoder) {
+    var string = this._decoder.write(d);
+    if (string.length) this.emit('data', string);
   } else {
-    var string = d.toString(this._encoding, 0, d.length);
-    this.emit('data', string);
+    this.emit('data', d);
   }
 };
 
@@ -874,7 +887,7 @@ WriteStream.prototype.write = function (data) {
     cb = arguments[arguments.length-1];
   }
 
-  if (data instanceof Buffer) {
+  if (Buffer.isBuffer(data)) {
     this._queue.push([fs.write, data, 0, data.length, null, cb]);
   } else {
     var encoding = 'utf8';
