@@ -9,7 +9,6 @@ if (debugLevel & 0x4) {
 }
 
 var net = require('net');
-var Utf8Decoder = require('utf8decoder').Utf8Decoder;
 var events = require('events');
 var Buffer = require('buffer').Buffer;
 
@@ -38,7 +37,7 @@ var parsers = new FreeList('parsers', 1000, function () {
 
   parser.onHeaderField = function (b, start, len) {
     var slice = b.toString('ascii', start, start+len).toLowerCase();
-    if (parser.value) {
+    if (parser.value != undefined) {
       parser.incoming._addHeaderLine(parser.field, parser.value);
       parser.field = null;
       parser.value = null;
@@ -60,7 +59,7 @@ var parsers = new FreeList('parsers', 1000, function () {
   };
 
   parser.onHeadersComplete = function (info) {
-    if (parser.field && parser.value) {
+    if (parser.field && (parser.value != undefined)) {
       parser.incoming._addHeaderLine(parser.field, parser.value);
     }
 
@@ -93,14 +92,12 @@ var parsers = new FreeList('parsers', 1000, function () {
 
   parser.onBody = function (b, start, len) {
     // TODO body encoding?
-    var enc = parser.incoming._encoding;
-    if (!enc) {
-      parser.incoming.emit('data', b.slice(start, start+len));
-    } else if (this._decoder) {
-      this._decoder.write(pool.slice(start, end));
+    var slice = b.slice(start, start+len);
+    if (parser.incoming._decoder) {
+      var string = parser.incoming._decoder.write(slice);
+      if (string.length) parser.incoming.emit('data', string);
     } else {
-      var string = b.toString(enc, start, start+len);
-      parser.incoming.emit('data', string);
+      parser.incoming.emit('data', slice);
     }
   };
 
@@ -217,18 +214,9 @@ IncomingMessage.prototype.setBodyEncoding = function (enc) {
   this.setEncoding(enc);
 };
 
-IncomingMessage.prototype.setEncoding = function (enc) {
-  // TODO check values, error out on bad, and deprecation message?
-  this._encoding = enc.toLowerCase();
-  if (this._encoding == 'utf-8' || this._encoding == 'utf8') {
-    this._decoder = new Utf8Decoder();
-    this._decoder.onString = function(str) {
-      this.emit('data', str);
-    };
-  } else if (this._decoder) {
-    delete this._decoder;
-  }
-
+IncomingMessage.prototype.setEncoding = function (encoding) {
+  var StringDecoder = require("string_decoder").StringDecoder; // lazy load
+  this._decoder = new StringDecoder(encoding);
 };
 
 IncomingMessage.prototype.pause = function () {
@@ -239,13 +227,35 @@ IncomingMessage.prototype.resume = function () {
   this.socket.resume();
 };
 
+// Add the given (field, value) pair to the message
+//
+// Per RFC2616, section 4.2 it is acceptable to join multiple instances of the
+// same header with a ', ' if the header in question supports specification of
+// multiple values this way. If not, we declare the first instance the winner
+// and drop the second. Extended header fields (those beginning with 'x-') are
+// always joined.
 IncomingMessage.prototype._addHeaderLine = function (field, value) {
-  if (field in this.headers) {
-    // TODO Certain headers like 'Content-Type' should not be concatinated.
-    // See https://www.google.com/reader/view/?tab=my#overview-page
-    this.headers[field] += ", " + value;
-  } else {
+  if (!(field in this.headers)) {
     this.headers[field] = value;
+    return;
+  }
+
+  // If this field already exists in the request, use duplicate-resolution
+  // logic from RFC2616.
+  switch (field) {
+  case 'accept':
+  case 'accept-charset':
+  case 'accept-encoding':
+  case 'accept-language':
+  case 'connection':
+  case 'cookie':
+    this.headers[field] += ', ' + value;
+    break;
+
+  default:
+    if (field[0] !== 'x' || field[1] !== '-') break;
+    this.headers[field] += ', ' + value;
+    break;
   }
 };
 
@@ -349,7 +359,7 @@ OutgoingMessage.prototype._storeHeader = function (firstLine, headers) {
 
   if (headers) {
     var keys = Object.keys(headers);
-    var isArray = (headers instanceof Array);
+    var isArray = (Array.isArray(headers));
     for (var i = 0, l = keys.length; i < l; i++) {
       var key = keys[i];
       if (isArray) {
@@ -423,7 +433,7 @@ OutgoingMessage.prototype.write = function (chunk, encoding) {
   }
 
   if (typeof chunk !== "string"
-      && !(chunk instanceof Buffer)
+      && !Buffer.isBuffer(chunk)
       && !Array.isArray(chunk)) {
     throw new TypeError("first argument must be a string, Array, or Buffer");
   }
@@ -863,11 +873,18 @@ function Client ( ) {
       self.destroy(ret);
     } else if (parser.incoming && parser.incoming.upgrade) {
       var bytesParsed = ret;
-      var upgradeHead = d.slice(start + bytesParsed, end - start);
-      parser.incoming.upgradeHead = upgradeHead;
-      var req = self._outgoing[0];
       self.ondata = null;
       self.onend = null
+
+      var req = self._outgoing[0];
+
+      var upgradeHead = d.slice(start + bytesParsed + 1, end);
+
+      if (self.listeners('upgrade').length) {
+        self.emit('upgrade', req, self, upgradeHead);
+      } else {
+        self.destroy();
+      }
     }
   };
 
@@ -1019,7 +1036,7 @@ exports.cat = function (url, encoding_, headers_) {
   var url = require("url").parse(url);
 
   var hasHost = false;
-  if (headers instanceof Array) {
+  if (Array.isArray(headers)) {
     for (var i = 0, l = headers.length; i < l; i++) {
       if (headers[i][0].toLowerCase() === 'host') {
         hasHost = true;
