@@ -2,6 +2,13 @@
 
 // Module
 
+
+// Set the environ variable NODE_MODULE_CONTEXT=1 to make node load all
+// modules in thier own context.
+var contextLoad = false;
+if (parseInt(process.env["NODE_MODULE_CONTEXTS"]) > 0) contextLoad = true;
+var Script;
+
 var internalModuleCache = {};
 var extensionCache = {};
 
@@ -15,7 +22,6 @@ function Module (id, parent) {
   } else {
     this.moduleCache = {};
   }
-  this.moduleCache[this.id] = this;
 
   this.filename = null;
   this.loaded = false;
@@ -44,7 +50,9 @@ function loadNative (id) {
   m.loaded = true;
   return m;
 }
+
 exports.requireNative = requireNative;
+
 function requireNative (id) {
   if (internalModuleCache[id]) return internalModuleCache[id].exports;
   if (!natives[id]) throw new Error('No such native module ' + id);
@@ -86,7 +94,7 @@ function existsSync (path) {
 
 
 
-var modulePaths = [path.join(process.installPrefix, "lib", "node")];
+var modulePaths = [path.join(process.execPath, "..", "..", "lib", "node")];
 
 if (process.env["HOME"]) {
   modulePaths.unshift(path.join(process.env["HOME"], ".node_libraries"));
@@ -134,7 +142,8 @@ function findModulePath (id, dirs, callback) {
     path.join(dir, id + ".js"),
     path.join(dir, id + ".node"),
     path.join(dir, id, "index.js"),
-    path.join(dir, id, "index.node")
+    path.join(dir, id, "index.node"),
+    path.join(dir, id)
   ];
 
   var ext;
@@ -179,7 +188,6 @@ function resolveModulePath(request, parent) {
   var id, paths;
   if (request.charAt(0) == "." && (request.charAt(1) == "/" || request.charAt(1) == ".")) {
     // Relative request
-    debug("RELATIVE: requested:" + request + " set ID to: "+id+" from "+parent.id);
 
     var exts = ['js', 'node'], ext;
     var extensions = Object.keys(extensionCache);
@@ -189,8 +197,14 @@ function resolveModulePath(request, parent) {
     }
 
     var parentIdPath = path.dirname(parent.id +
-      (path.basename(parent.filename).match(new RegExp('^index\\.(' + exts.join('|') + ')$')) ? "/" : ""));
+      (path.basename(parent.filename).match(new RegExp('^index\\.(' + exts.join('|') + ')$')) ? "/." : ""));
     id = path.join(parentIdPath, request);
+    // make sure require('./path') and require('path') get distinct ids, even
+    // when called from the toplevel js file
+    if (parentIdPath == '.' && ! id.match(new RegExp('/'))) {
+      id = './' + id;
+    }
+    debug("RELATIVE: requested:" + request + " set ID to: "+id+" from "+parent.id);
     paths = [path.dirname(parent.filename)];
   } else {
     id = request;
@@ -209,52 +223,48 @@ function loadModule (request, parent, callback) {
 
   debug("loadModule REQUEST  " + (request) + " parent: " + parent.id);
 
-  var cachedModule = internalModuleCache[id] || parent.moduleCache[id];
-
-  if (!cachedModule) {
-    // Try to compile from native modules
-    if (natives[id]) {
-      debug('load native module ' + id);
-      cachedModule = loadNative(id);
-    }
+  // native modules always take precedence.
+  var cachedNative = internalModuleCache[id];
+  if (cachedNative) {
+    return callback ? callback(null, cachedNative.exports) : cachedNative.exports;
+  }
+  if (natives[id]) {
+    debug('load native module ' + id);
+    var nativeMod = loadNative(id);
+    return callback ? callback(null, nativeMod.exports) : nativeMod.exports;
   }
 
-  if (cachedModule) {
-    debug("found  " + JSON.stringify(id) + " in cache");
-    if (callback) {
-      callback(null, cachedModule.exports);
-    } else {
-      return cachedModule.exports;
+  // look up the filename first, since that's the cache key.
+  debug("looking for " + JSON.stringify(id) + " in " + JSON.stringify(paths));
+  if (!callback) {
+    // sync
+    var filename = findModulePath(request, paths);
+    if (!filename) {
+      throw new Error("Cannot find module '" + request + "'");
     }
 
-  } else {
-    // Not in cache
-    debug("looking for " + JSON.stringify(id) + " in " + JSON.stringify(paths));
+    var cachedModule = parent.moduleCache[filename];
+    if (cachedModule) return cachedModule.exports;
 
-    if (!callback) {
-      // sync
-      var filename = findModulePath(request, paths);
-      if (!filename) {
-        throw new Error("Cannot find module '" + request + "'");
-      } else {
-        var module = new Module(id, parent);
-        module.loadSync(filename);
-        return module.exports;
-      }
-
-    } else {
-      // async
-      findModulePath(request, paths, function (filename) {
-        if (!filename) {
-          var err = new Error("Cannot find module '" + request + "'");
-          callback(err);
-        } else {
-          var module = new Module(id, parent);
-          module.load(filename, callback);
-        }
-      });
-    }
+    var module = new Module(id, parent);
+    module.moduleCache[filename] = module;
+    module.loadSync(filename);
+    return module.exports;
   }
+  // async
+  findModulePath(request, paths, function (filename) {
+    if (!filename) {
+      var err = new Error("Cannot find module '" + request + "'");
+      return callback(err);
+    }
+
+    var cachedModule = parent.moduleCache[filename];
+    if (cachedModule) return callback(null, cachedModule.exports);
+
+    var module = new Module(id, parent);
+    module.moduleCache[filename] = module;
+    module.load(filename, callback);
+  });
 };
 
 
@@ -369,21 +379,51 @@ Module.prototype._compile = function (content, filename) {
   require.main = process.mainModule;
   require.registerExtension = registerExtension;
 
+  var dirname = path.dirname(filename);
 
-  if ('string' === typeof content) {
-    // create wrapper function
-    var wrapper = "(function (exports, require, module, __filename, __dirname) { "
-                + content
-                + "\n});";
+  if (contextLoad) {
+    if (!Script) Script = Script = process.binding('evals').Script;
+
+    if (self.id !== ".") {
+      debug('load submodule');
+      // not root module
+      var sandbox = {};
+      for (var k in global) {
+        sandbox[k] = global[k];
+      }
+      sandbox.require     = require;
+      sandbox.exports     = self.exports;
+      sandbox.__filename  = filename;
+      sandbox.__dirname   = dirname;
+      sandbox.module      = self;
+      sandbox.root        = global;
+
+      Script.runInNewContext(content, sandbox, filename);
+
+    } else {
+      debug('load root module');
+      // root module
+      global.require    = require;
+      global.exports    = self.exports;
+      global.__filename = filename;
+      global.__dirname  = dirname;
+      global.module     = self;
+      global.root       = global;
+      Script.runInThisContext(content, filename);
+    }
+
+  } else {
+    if ('string' === typeof content) {
+      // create wrapper function
+      var wrapper = "(function (exports, require, module, __filename, __dirname) { "
+                  + content
+                  + "\n});";
 
       var compiledWrapper = process.compile(wrapper, filename);
-      var dirName = path.dirname(filename);
-      if (filename === process.argv[1]) {
-        process.checkBreak();
-      }
-      compiledWrapper.apply(self.exports, [self.exports, require, self, filename, dirName]);
-  } else {
-    self.exports = content;
+      compiledWrapper.apply(self.exports, [self.exports, require, self, filename, dirname]);
+    } else {
+      self.exports = content;
+    }
   }
 };
 
