@@ -1,6 +1,5 @@
 var sys = require("sys");
 var fs = require("fs");
-var Utf8Decoder = require("utf8decoder").Utf8Decoder;
 var events = require("events");
 var dns = require('dns');
 
@@ -294,13 +293,21 @@ function setImplmentationMethods (self) {
     self._readImpl = function(buf, off, len, calledByIOWatcher) {
       var bytesRead = recvMsg(self.fd, buf, off, len);
 
-      // Do not emit this in the same stack, otherwise we risk corrupting
-      // our buffer pool which is full of read data, but has not had
-      // had its pointers updated just yet.
+      // Do not emit this in the same stack, otherwise we risk corrupting our
+      // buffer pool which is full of read data, but has not had had its
+      // pointers updated just yet.
+      //
+      // Save off recvMsg.fd in a closure so that, when we emit it later, we're
+      // emitting the same value that we see now. Otherwise, we can end up
+      // calling emit() after recvMsg() has been called again and end up
+      // emitting null (or another FD).
       if (recvMsg.fd !== null) {
-        process.nextTick(function() {
-          self.emit('fd', recvMsg.fd);
-        });
+        (function () {
+          var fd = recvMsg.fd;
+          process.nextTick(function() {
+            self.emit('fd', fd);
+          });
+        })();
       }
 
       return bytesRead;
@@ -311,7 +318,9 @@ function setImplmentationMethods (self) {
       //      by the kernel for some reason. Otherwise, we'd just
       //      fast-path return here.
 
-      return write(self.fd, buf, off, len, fd, flags);
+      // Drop 'fd' and 'flags' as these are not supported by the write(2)
+      // system call
+      return write(self.fd, buf, off, len);
     };
 
     self._readImpl = function(buf, off, len, calledByIOWatcher) {
@@ -356,7 +365,7 @@ function setImplmentationMethods (self) {
 
       return bytesWritten;
     };
-    
+
     var oldRead = self._readImpl;
     self._readImpl = function(buf, off, len, calledByIOWatcher) {
       assert(self.secure);
@@ -492,20 +501,20 @@ function initStream (self) {
       var end = pool.used + bytesRead;
       pool.used += bytesRead;
 
-      if (!self._encoding) {
+      if (self._decoder) {
+        // emit String
+        var string = self._decoder.write(pool.slice(start, end));
+        if (string.length) self.emit('data', string);
+      } else {
+        // emit buffer
         if (self._events && self._events['data']) {
           // emit a slice
           self.emit('data', pool.slice(start, end));
         }
-
-        // Optimization: emit the original buffer with end points
-        if (self.ondata) self.ondata(pool, start, end);
-      } else if (this._decoder) {
-        this._decoder.write(pool.slice(start, end));
-      } else {
-        var string = pool.toString(self._encoding, start, end);
-        self.emit('data', string);
       }
+
+      // Optimization: emit the original buffer with end points
+      if (self.ondata) self.ondata(pool, start, end);
     }
   };
   self.readable = false;
@@ -645,13 +654,11 @@ Object.defineProperty(Stream.prototype, 'readyState', {
 // Returns true if all the data was flushed to socket. Returns false if
 // something was queued. If data was queued, then the "drain" event will
 // signal when it has been finally flushed to socket.
-//
-// XXX: Caller cannot close the given fd until the stream has drained.
 Stream.prototype.write = function (data, encoding, fd) {
   if (this._writeQueue && this._writeQueue.length) {
     // Slow. There is already a write queue, so let's append to it.
     if (this._writeQueueLast() === END_OF_FILE) {
-      throw new Error('Stream.close() called already; cannot write.');
+      throw new Error('Stream.end() called already; cannot write.');
     }
 
     if (typeof data == 'string' &&
@@ -820,18 +827,9 @@ Stream.prototype._writeQueueLast = function () {
 };
 
 
-Stream.prototype.setEncoding = function (enc) {
-  var self = this;
-  // TODO check values, error out on bad, and deprecation message?
-  this._encoding = enc.toLowerCase();
-  if (this._encoding == 'utf-8' || this._encoding == 'utf8') {
-    this._decoder = new Utf8Decoder();
-    this._decoder.onString = function(str) {
-      self.emit('data', str);
-    };
-  } else if (this._decoder) {
-    delete this._decoder;
-  }
+Stream.prototype.setEncoding = function (encoding) {
+  var StringDecoder = require("string_decoder").StringDecoder; // lazy load
+  this._decoder = new StringDecoder(encoding);
 };
 
 
@@ -1112,6 +1110,11 @@ Server.prototype.listen = function () {
   var self = this;
   if (self.fd) throw new Error('Server already opened');
 
+  var lastArg = arguments[arguments.length - 1];
+  if (typeof lastArg == 'function') {
+    self.addListener('listening', lastArg);
+  }
+
   if (!isPort(arguments[0])) {
     // the first argument specifies a path
     self.fd = socket('unix');
