@@ -38,11 +38,9 @@ exports.normalize = function (path, keepBlanks) {
 };
 
 exports.dirname = function (path) {
-  // Can't strip trailing slashes since module.js incorrectly thinks
-  // dirname('/a/b/') should yield '/a/b' instead of '/a'.
-  // if (path.length > 1 && '/' === path[path.length-1]) {
-  //  path = path.replace(/\/+$/, '');
-  // }
+  if (path.length > 1 && '/' === path[path.length-1]) {
+    path = path.replace(/\/+$/, '');
+  }
   var lastSlash = path.lastIndexOf('/');
   switch (lastSlash) {
     case -1:
@@ -79,3 +77,12 @@ exports.exists = function (path, callback) {
     if (callback) callback(err ? false : true);
   });
 };
+
+exports.existsSync = function (path) {
+  try {
+    process.binding('fs').stat(path)
+    return true;
+  } catch(e){
+    return false;
+  }
+};
