@@ -10,13 +10,14 @@
 //   repl.start("node via TCP socket> ", socket);
 // }).listen(5001);
 
-// repl.start("node > ").scope.foo = "stdin is fun";  // expose foo to repl scope
+// repl.start("node > ").context.foo = "stdin is fun";  // expose foo to repl context
 
 var sys = require('sys');
-var evalcx = process.binding('evals').Script.runInNewContext;
+var Script = process.binding('evals').Script;
+var evalcx = Script.runInContext;
 var path = require("path");
 var rl = require('readline');
-var scope;
+var context;
 
 function cwdRequire (id) {
   if (id.match(/^\.\.\//) || id.match(/^\.\//)) {
@@ -24,12 +25,15 @@ function cwdRequire (id) {
   }
   return require(id);
 }
+Object.keys(require).forEach(function (k) {
+  cwdRequire[k] = require[k];
+});
 
-function setScope (self) {
-  scope = {};
-  for (var i in global) scope[i] = global[i];
-  scope.module = module;
-  scope.require = cwdRequire;
+function resetContext() {
+  context = Script.createContext();
+  for (var i in global) context[i] = global[i];
+  context.module = module;
+  context.require = cwdRequire;
 }
 
 
@@ -38,8 +42,8 @@ exports.writer = sys.inspect;
 
 function REPLServer(prompt, stream) {
   var self = this;
-  if (!scope) setScope();
-  self.scope = scope;
+  if (!context) resetContext();
+  self.context = context;
   self.buffered_cmd = '';
 
   self.stream = stream || process.openStdin();
@@ -67,10 +71,10 @@ function REPLServer(prompt, stream) {
       // This try is for determining if the command is complete, or should
       // continue onto the next line.
       try {
-        // Use evalcx to supply the global scope
-        var ret = evalcx(self.buffered_cmd, scope, "repl");
+        // Use evalcx to supply the global context
+        var ret = evalcx(self.buffered_cmd, context, "repl");
         if (ret !== undefined) {
-          scope._ = ret;
+          context._ = ret;
           flushed = self.stream.write(exports.writer(ret) + "\n");
         }
 
@@ -133,23 +137,23 @@ REPLServer.prototype.readline = function (cmd) {
 
 /**
  * Used to parse and execute the Node REPL commands.
- * 
+ *
  * @param {cmd} cmd The command entered to check
- * @returns {Boolean} If true it means don't continue parsing the command 
+ * @returns {Boolean} If true it means don't continue parsing the command
  */
 
 REPLServer.prototype.parseREPLKeyword = function (cmd) {
   var self = this;
-  
+
   switch (cmd) {
   case ".break":
     self.buffered_cmd = '';
     self.displayPrompt();
     return true;
   case ".clear":
-    self.stream.write("Clearing Scope...\n");
+    self.stream.write("Clearing context...\n");
     self.buffered_cmd = '';
-    setScope();
+    resetContext();
     self.displayPrompt();
     return true;
   case ".exit":
@@ -157,7 +161,7 @@ REPLServer.prototype.parseREPLKeyword = function (cmd) {
     return true;
   case ".help":
     self.stream.write(".break\tSometimes you get stuck in a place you can't get out... This will get you out.\n");
-    self.stream.write(".clear\tBreak, and also clear the local scope.\n");
+    self.stream.write(".clear\tBreak, and also clear the local context.\n");
     self.stream.write(".exit\tExit the prompt\n");
     self.stream.write(".help\tShow repl options\n");
     self.displayPrompt();
@@ -167,7 +171,7 @@ REPLServer.prototype.parseREPLKeyword = function (cmd) {
 };
 
 function trimWhitespace (cmd) {
-  var trimmer = /^\s*(.+)\s*$/m, 
+  var trimmer = /^\s*(.+)\s*$/m,
     matches = trimmer.exec(cmd);
 
   if (matches && matches.length === 2) {
@@ -177,28 +181,28 @@ function trimWhitespace (cmd) {
 
 /**
  * Converts commands that use var and function <name>() to use the
- * local exports.scope when evaled. This provides a local scope
+ * local exports.context when evaled. This provides a local context
  * on the REPL.
- * 
+ *
  * @param {String} cmd The cmd to convert
  * @returns {String} The converted command
  */
-REPLServer.prototype.convertToScope = function (cmd) {
+REPLServer.prototype.convertToContext = function (cmd) {
   var self = this, matches,
     scopeVar = /^\s*var\s*([_\w\$]+)(.*)$/m,
     scopeFunc = /^\s*function\s*([_\w\$]+)/;
-  
-  // Replaces: var foo = "bar";  with: self.scope.foo = bar;
+
+  // Replaces: var foo = "bar";  with: self.context.foo = bar;
   matches = scopeVar.exec(cmd);
   if (matches && matches.length === 3) {
-    return "self.scope." + matches[1] + matches[2];
+    return "self.context." + matches[1] + matches[2];
   }
-  
+
   // Replaces: function foo() {};  with: foo = function foo() {};
   matches = scopeFunc.exec(self.buffered_cmd);
   if (matches && matches.length === 2) {
     return matches[1] + " = " + self.buffered_cmd;
   }
-  
+
   return cmd;
 };
