@@ -29,10 +29,12 @@ process.assert = function (x, msg) {
   if (!x) throw new Error(msg || "assertion error");
 };
 
+var writeError = process.binding('stdio').writeError;
+
 var evalcxMsg;
 process.evalcx = function () {
   if (!evalcxMsg) {
-    process.binding('stdio').writeError(evalcxMsg =
+    writeError(evalcxMsg =
       "process.evalcx is deprecated. Use Script.runInNewContext instead.\n");
   }
   return process.binding('evals').Script
@@ -187,6 +189,48 @@ process.openStdin = function () {
 };
 
 
+// console object
+var formatRegExp = /%[sdj]/g;
+function format (f) {
+  var i = 1;
+  var args = arguments;
+  return String(f).replace(formatRegExp, function (x) {
+    switch (x) {
+      case '%s': return args[i++];
+      case '%d': return +args[i++];
+      case '%j': return JSON.stringify(args[i++]);
+      default:
+        return x;
+    }
+  });
+}
+
+global.console = {};
+
+global.console.log = function () {
+  process.stdout.write(format.apply(this, arguments) + '\n');
+};
+
+global.console.info = global.console.log;
+
+global.console.warn = function () {
+  writeError(format.apply(this, arguments) + '\n');
+};
+
+global.console.error = global.console.warn;
+
+global.console.dir = function(object){
+  var sys = module.requireNative('sys');
+  process.stdout.write(sys.inspect(object) + '\n');
+}
+
+global.console.assert = function(expression){
+  if(!expression){
+    var arr = Array.prototype.slice.call(arguments, 1);
+    process.assert(false, format.apply(this, arr));
+  }
+}
+
 process.exit = function (code) {
   process.emit("exit");
   process.reallyExit(code);
@@ -209,7 +253,7 @@ if (process.argv[1]) {
 } else {
   // No arguments, run the repl
   var repl = module.requireNative('repl');
-  process.stdout.write("Type '.help' for options.\n");
+  console.log("Type '.help' for options.");
   repl.start();
 }
 
