@@ -14,6 +14,8 @@
 #include <fcntl.h>
 #include <arpa/inet.h> /* inet_pton */
 
+#include <netdb.h>
+
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 
@@ -33,6 +35,8 @@
 #include <sys/uio.h>
 #endif
 
+#define ARRAY_SIZE(a) (sizeof(a) / sizeof(*(a)))
+
 
 namespace node {
 
@@ -42,6 +46,7 @@ static Persistent<String> errno_symbol;
 static Persistent<String> syscall_symbol;
 
 static Persistent<String> fd_symbol;
+static Persistent<String> size_symbol;
 static Persistent<String> address_symbol;
 static Persistent<String> port_symbol;
 static Persistent<String> type_symbol;
@@ -58,8 +63,6 @@ static Persistent<FunctionTemplate> recv_msg_template;
           String::New("Bad file descriptor argument"))); \
   }
 
-
-
 static inline bool SetCloseOnExec(int fd) {
   return (fcntl(fd, F_SETFD, FD_CLOEXEC) != -1);
 }
@@ -71,6 +74,8 @@ static inline bool SetNonBlock(int fd) {
 
 
 static inline bool SetSockFlags(int fd) {
+  int flags = 1;
+  setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (void *)&flags, sizeof(flags));
   return SetNonBlock(fd) && SetCloseOnExec(fd);
 }
 
@@ -147,7 +152,16 @@ static Handle<Value> Socket(const Arguments& args) {
     } else if (0 == strcasecmp(*t, "UNIX")) {
       domain = PF_UNIX;
       type = SOCK_STREAM;
+    } else if (0 == strcasecmp(*t, "UNIX_DGRAM")) {
+      domain = PF_UNIX;
+      type = SOCK_DGRAM;
     } else if (0 == strcasecmp(*t, "UDP")) {
+      domain = PF_INET;
+      type = SOCK_DGRAM;
+    } else if (0 == strcasecmp(*t, "UDP4")) {
+      domain = PF_INET;
+      type = SOCK_DGRAM;
+    } else if (0 == strcasecmp(*t, "UDP6")) {
       domain = PF_INET6;
       type = SOCK_DGRAM;
     } else {
@@ -185,13 +199,13 @@ static inline Handle<Value> ParseAddressArgs(Handle<Value> first,
     // UNIX
     String::Utf8Value path(first->ToString());
 
-    if (path.length() > sizeof un.sun_path) {
+    if (path.length() >= ARRAY_SIZE(un.sun_path)) {
       return Exception::Error(String::New("Socket path too long"));
     }
 
-    memset(&un, 0, sizeof un);
     un.sun_family = AF_UNIX;
-    strcpy(un.sun_path, *path);
+    strncpy(un.sun_path, *path, ARRAY_SIZE(un.sun_path) - 1);
+    un.sun_path[ARRAY_SIZE(un.sun_path) - 1] = '\0';
 
     addr = (struct sockaddr*)&un;
     addrlen = path.length() + sizeof(un.sun_family) + 1;
@@ -202,7 +216,7 @@ static inline Handle<Value> ParseAddressArgs(Handle<Value> first,
     memset(&in6, 0, sizeof in6);
 
     int port = first->Int32Value();
-    in.sin_port = in6.sin6_port = htons(port);  
+    in.sin_port = in6.sin6_port = htons(port);
     in.sin_family = AF_INET;
     in6.sin6_family = AF_INET6;
 
@@ -229,7 +243,7 @@ static inline Handle<Value> ParseAddressArgs(Handle<Value> first,
 }
 
 
-// Bind with UNIX 
+// Bind with UNIX
 //   t.bind(fd, "/tmp/socket")
 // Bind with TCP
 //   t.bind(fd, 80, "192.168.11.2")
@@ -336,6 +350,12 @@ static Handle<Value> Connect(const Arguments& args) {
   return Undefined();
 }
 
+// Mac's SUN_LEN is broken
+#if defined(__APPLE__)
+# define SUN_LEN(ptr) ((ptr)->sun_len-2)
+#elif !defined(SUN_LEN)
+# define SUN_LEN(ptr) strlen((ptr)->sun_path)
+#endif
 
 #define ADDRESS_TO_JS(info, address_storage) \
 do { \
@@ -343,6 +363,7 @@ do { \
   int port; \
   struct sockaddr_in *a4; \
   struct sockaddr_in6 *a6; \
+  struct sockaddr_un *au; \
   switch ((address_storage).ss_family) { \
     case AF_INET6: \
       a6 = (struct sockaddr_in6*)&(address_storage); \
@@ -358,6 +379,17 @@ do { \
       (info)->Set(address_symbol, String::New(ip)); \
       (info)->Set(port_symbol, Integer::New(port)); \
       break; \
+    case AF_UNIX: \
+      au = (struct sockaddr_un*)&(address_storage); \
+      char un_path[105]; \
+      size_t len; \
+      len = SUN_LEN(au); \
+      strncpy(un_path, au->sun_path, len); \
+      un_path[len] = 0; \
+      (info)->Set(address_symbol, String::New(un_path)); \
+      break; \
+    default: \
+      (info)->Set(address_symbol, String::New("")); \
   } \
 } while (0)
 
@@ -474,7 +506,7 @@ static Handle<Value> SocketError(const Arguments& args) {
     return ThrowException(ErrnoException(errno, "getsockopt"));
   }
 
-  return scope.Close(Integer::New(error)); 
+  return scope.Close(Integer::New(error));
 }
 
 
@@ -520,6 +552,63 @@ static Handle<Value> Read(const Arguments& args) {
   return scope.Close(Integer::New(bytes_read));
 }
 
+//  var info = t.recvfrom(fd, buffer, offset, length, flags);
+//    info.size // bytes read
+//    info.port // from port
+//    info.address // from address
+//  returns null on EAGAIN or EINTR, raises an exception on all other errors
+//  returns object otherwise
+static Handle<Value> RecvFrom(const Arguments& args) {
+  HandleScope scope;
+
+  if (args.Length() < 5) {
+    return ThrowException(Exception::TypeError(
+          String::New("Takes 5 parameters")));
+  }
+
+  FD_ARG(args[0])
+
+  if (!Buffer::HasInstance(args[1])) {
+    return ThrowException(Exception::TypeError(
+          String::New("Second argument should be a buffer")));
+  }
+
+  Buffer * buffer = ObjectWrap::Unwrap<Buffer>(args[1]->ToObject());
+
+  size_t off = args[2]->Int32Value();
+  if (off >= buffer->length()) {
+    return ThrowException(Exception::Error(
+          String::New("Offset is out of bounds")));
+  }
+
+  size_t len = args[3]->Int32Value();
+  if (off + len > buffer->length()) {
+    return ThrowException(Exception::Error(
+          String::New("Length is extends beyond buffer")));
+  }
+
+  int flags = args[4]->Int32Value();
+
+  struct sockaddr_storage address_storage;
+  socklen_t addrlen = sizeof(struct sockaddr_storage);
+
+  ssize_t bytes_read = recvfrom(fd, (char*)buffer->data() + off, len, flags,
+                                (struct sockaddr*) &address_storage, &addrlen);
+
+  if (bytes_read < 0) {
+    if (errno == EAGAIN || errno == EINTR) return Null();
+    return ThrowException(ErrnoException(errno, "read"));
+  }
+
+  Local<Object> info = Object::New();
+
+  info->Set(size_symbol, Integer::New(bytes_read));
+
+  ADDRESS_TO_JS(info, address_storage);
+
+  return scope.Close(info);
+}
+
 
 // bytesRead = t.recvMsg(fd, buffer, offset, length)
 // if (recvMsg.fd) {
@@ -559,6 +648,7 @@ static Handle<Value> RecvMsg(const Arguments& args) {
   iov[0].iov_len = len;
 
   struct msghdr msg;
+  msg.msg_flags = 0;
   msg.msg_iov = iov;
   msg.msg_iovlen = 1;
   msg.msg_name = NULL;
@@ -584,7 +674,7 @@ static Handle<Value> RecvMsg(const Arguments& args) {
   //
   // XXX: Some implementations can send multiple file descriptors in a
   //      single message. We should be using CMSG_NXTHDR() to walk the
-  //      chain to get at them all. This would require changing the 
+  //      chain to get at them all. This would require changing the
   //      API to hand these back up the caller, is a pain.
 
   int received_fd = -1;
@@ -627,7 +717,7 @@ static Handle<Value> Write(const Arguments& args) {
 
   FD_ARG(args[0])
 
-  if (!Buffer::HasInstance(args[1])) { 
+  if (!Buffer::HasInstance(args[1])) {
     return ThrowException(Exception::TypeError(
           String::New("Second argument should be a buffer")));
   }
@@ -780,6 +870,93 @@ static Handle<Value> SendMsg(const Arguments& args) {
   return scope.Close(Integer::New(written));
 }
 
+// var bytes = sendto(fd, buf, off, len, flags, destination port, desitnation address);
+//
+// Write a buffer with optional offset and length to the given file
+// descriptor. Note that we refuse to send 0 bytes.
+//
+// The 'fd' parameter is a numerical file descriptor, or the undefined value
+// to send none.
+//
+// The 'flags' parameter is a number representing a bitmask of MSG_* values.
+// This is passed directly to sendmsg().
+//
+// The destination port can either be an int port, or a path.
+//
+// Returns null on EAGAIN or EINTR, raises an exception on all other errors
+static Handle<Value> SendTo(const Arguments& args) {
+  HandleScope scope;
+
+  if (args.Length() < 5) {
+    return ThrowException(Exception::TypeError(
+          String::New("Takes 5 or 6 parameters")));
+  }
+
+  // The first argument should be a file descriptor
+  FD_ARG(args[0])
+
+  // Grab the actul data to be written
+  if (!Buffer::HasInstance(args[1])) {
+    return ThrowException(Exception::TypeError(
+      String::New("Expected either a string or a buffer")));
+  }
+
+  Buffer *buf = ObjectWrap::Unwrap<Buffer>(args[1]->ToObject());
+
+  size_t offset = 0;
+  if (args.Length() >= 3 && !args[2]->IsUndefined()) {
+    if (!args[2]->IsUint32()) {
+      return ThrowException(Exception::TypeError(
+        String::New("Expected unsigned integer for offset")));
+    }
+
+    offset = args[2]->Uint32Value();
+    if (offset >= buf->length()) {
+      return ThrowException(Exception::Error(
+        String::New("Offset into buffer too large")));
+    }
+  }
+
+  size_t length = buf->length() - offset;
+  if (args.Length() >= 4 && !args[3]->IsUndefined()) {
+    if (!args[3]->IsUint32()) {
+      return ThrowException(Exception::TypeError(
+        String::New("Expected unsigned integer for length")));
+    }
+
+    length = args[3]->Uint32Value();
+    if (offset + length > buf->length()) {
+      return ThrowException(Exception::Error(
+        String::New("offset + length beyond buffer length")));
+    }
+  }
+
+  int flags = 0;
+  if (args.Length() >= 5 && !args[4]->IsUndefined()) {
+    if (!args[4]->IsUint32()) {
+      return ThrowException(Exception::TypeError(
+        String::New("Expected unsigned integer for a flags argument")));
+    }
+
+    flags = args[4]->Uint32Value();
+  }
+
+  Handle<Value> error = ParseAddressArgs(args[5], args[6], false);
+  if (!error.IsEmpty()) return ThrowException(error);
+
+  ssize_t written = sendto(fd, buf->data() + offset, length, flags, addr, addrlen);
+
+  if (written < 0) {
+    if (errno == EAGAIN || errno == EINTR) return Null();
+    return ThrowException(ErrnoException(errno, "sendmsg"));
+  }
+
+  /* Note that the FD isn't explicitly closed here, this
+   * happens in the JS */
+
+  return scope.Close(Integer::New(written));
+}
+
 
 // Probably only works for Linux TCP sockets?
 // Returns the amount of data on the read queue.
@@ -814,7 +991,6 @@ static Handle<Value> SetNoDelay(const Arguments& args) {
   return Undefined();
 }
 
-
 static Handle<Value> SetKeepAlive(const Arguments& args) {
   int r;
   HandleScope scope;
@@ -846,10 +1022,214 @@ static Handle<Value> SetKeepAlive(const Arguments& args) {
   return Undefined();
 }
 
+static Handle<Value> SetBroadcast(const Arguments& args) {
+  int flags, r;
+  HandleScope scope;
+
+  FD_ARG(args[0])
+
+  flags = args[1]->IsFalse() ? 0 : 1;
+  r = setsockopt(fd, SOL_SOCKET, SO_BROADCAST, (void *)&flags, sizeof(flags));
+
+  if (r < 0) {
+    return ThrowException(ErrnoException(errno, "setsockopt"));
+  } else {
+    return scope.Close(Integer::New(flags));
+  }
+}
+
+static Handle<Value> SetTTL(const Arguments& args) {
+  HandleScope scope;
+
+  if (args.Length() != 2) {
+    return ThrowException(Exception::TypeError(
+      String::New("Takes exactly two arguments: fd, new TTL")));
+  }
+
+  FD_ARG(args[0]);
+
+  if (! args[1]->IsInt32()) {
+    return ThrowException(Exception::TypeError(
+      String::New("Argument must be a number")));
+  }
+  
+  int newttl = args[1]->Int32Value();
+  if (newttl < 1 || newttl > 255) {
+    return ThrowException(Exception::TypeError(
+      String::New("new TTL must be between 1 and 255")));
+  }
+
+  int r = setsockopt(fd, IPPROTO_IP, IP_TTL, (void *)&newttl, sizeof(newttl));
+
+  if (r < 0) {
+    return ThrowException(ErrnoException(errno, "setsockopt"));
+  } else {
+    return scope.Close(Integer::New(newttl));
+  }
+}
+
+
+//
+// G E T A D D R I N F O
+//
+
+
+struct resolve_request {
+  Persistent<Function> cb;
+  struct addrinfo *address_list;
+  int ai_family; // AF_INET or AF_INET6
+  char hostname[1];
+};
+
+#ifndef EAI_NODATA // EAI_NODATA is deprecated, FreeBSD already thrown it away in favor of EAI_NONAME
+#define EAI_NODATA EAI_NONAME
+#endif
+
+static int AfterResolve(eio_req *req) {
+  ev_unref(EV_DEFAULT_UC);
+
+  struct resolve_request * rreq = (struct resolve_request *)(req->data);
+
+  HandleScope scope;
+  Local<Value> argv[2];
+
+  if (req->result != 0) {
+    argv[1] = Array::New();
+    if (req->result == EAI_NODATA) {
+      argv[0] = Local<Value>::New(Null());
+    } else {
+      argv[0] = ErrnoException(req->result,
+                               "getaddrinfo",
+                               gai_strerror(req->result));
+    }
+  } else {
+    struct addrinfo *address;
+    int n = 0;
+
+    for (address = rreq->address_list; address; address = address->ai_next) { n++; }
+
+    Local<Array> results = Array::New(n);
+
+    char ip[INET6_ADDRSTRLEN];
+    const char *addr;
+
+    n = 0;
+    address = rreq->address_list;
+    while (address) {
+      assert(address->ai_socktype == SOCK_STREAM);
+      assert(address->ai_family == AF_INET || address->ai_family == AF_INET6);
+      addr = ( address->ai_family == AF_INET
+             ? (char *) &((struct sockaddr_in *) address->ai_addr)->sin_addr
+             : (char *) &((struct sockaddr_in6 *) address->ai_addr)->sin6_addr
+             );
+      const char *c = inet_ntop(address->ai_family, addr, ip, INET6_ADDRSTRLEN);
+      Local<String> s = String::New(c);
+      results->Set(Integer::New(n), s);
+
+      n++;
+      address = address->ai_next;
+    }
+
+    argv[0] = Local<Value>::New(Null());
+    argv[1] = results;
+  }
+
+  TryCatch try_catch;
+
+  rreq->cb->Call(Context::GetCurrent()->Global(), 2, argv);
+
+  if (try_catch.HasCaught()) {
+    FatalException(try_catch);
+  }
+
+  if (rreq->address_list) freeaddrinfo(rreq->address_list);
+  rreq->cb.Dispose(); // Dispose of the persistent handle
+  free(rreq);
+
+  return 0;
+}
+
+
+static int Resolve(eio_req *req) {
+  // Note: this function is executed in the thread pool! CAREFUL
+  struct resolve_request * rreq = (struct resolve_request *) req->data;
+
+  struct addrinfo hints;
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family = rreq->ai_family;
+  hints.ai_socktype = SOCK_STREAM;
+
+  req->result = getaddrinfo((char*)rreq->hostname,
+                            NULL,
+                            &hints,
+                            &(rreq->address_list));
+  return 0;
+}
+
+
+static Handle<Value> GetAddrInfo(const Arguments& args) {
+  HandleScope scope;
+
+  String::Utf8Value hostname(args[0]->ToString());
+
+  int type = args[1]->Int32Value();
+  int fam = AF_INET;
+  switch (type) {
+    case 4:
+      fam = AF_INET;
+      break;
+    case 6:
+      fam = AF_INET6;
+      break;
+    default:
+      return ThrowException(Exception::TypeError(
+            String::New("Second argument must be an integer 4 or 6")));
+  }
+
+  if (!args[2]->IsFunction()) {
+    return ThrowException(Exception::TypeError(
+          String::New("Thrid argument must be a callback")));
+  }
+
+  Local<Function> cb = Local<Function>::Cast(args[2]);
+
+  struct resolve_request *rreq = (struct resolve_request *)
+    calloc(1, sizeof(struct resolve_request) + hostname.length() + 1);
+
+  if (!rreq) {
+    V8::LowMemoryNotification();
+    return ThrowException(Exception::Error(
+          String::New("Could not allocate enough memory")));
+  }
+
+  strncpy(rreq->hostname, *hostname, hostname.length() + 1);
+  rreq->cb = Persistent<Function>::New(cb);
+  rreq->ai_family = fam;
+
+  // For the moment I will do DNS lookups in the eio thread pool. This is
+  // sub-optimal and cannot handle massive numbers of requests.
+  //
+  // (One particularly annoying problem is that the pthread stack size needs
+  // to be increased dramatically to handle getaddrinfo() see X_STACKSIZE in
+  // wscript ).
+  //
+  // In the future I will move to a system using c-ares:
+  // http://lists.schmorp.de/pipermail/libev/2009q1/000632.html
+  eio_custom(Resolve, EIO_PRI_DEFAULT, AfterResolve, rreq);
+
+  // There will not be any active watchers from this object on the event
+  // loop while getaddrinfo() runs. If the only thing happening in the
+  // script was this hostname resolution, then the event loop would drop
+  // out. Thus we need to add ev_ref() until AfterResolve().
+  ev_ref(EV_DEFAULT_UC);
+
+  return Undefined();
+}
+
 
 static Handle<Value> IsIP(const Arguments& args) {
   HandleScope scope;
-  
+
   if (!args[0]->IsString()) {
     return scope.Close(Integer::New(4));
   }
@@ -878,7 +1258,7 @@ static Handle<Value> CreateErrnoException(const Arguments& args) {
   int errorno = args[0]->Int32Value();
   String::Utf8Value syscall(args[1]->ToString());
 
-  Local<Value> exception = ErrnoException(errorno, *syscall); 
+  Local<Value> exception = ErrnoException(errorno, *syscall);
 
   return scope.Close(exception);
 }
@@ -891,6 +1271,8 @@ void InitNet(Handle<Object> target) {
   NODE_SET_METHOD(target, "read", Read);
 
   NODE_SET_METHOD(target, "sendMsg", SendMsg);
+  NODE_SET_METHOD(target, "recvfrom", RecvFrom);
+  NODE_SET_METHOD(target, "sendto", SendTo);
 
   recv_msg_template =
       Persistent<FunctionTemplate>::New(FunctionTemplate::New(RecvMsg));
@@ -909,9 +1291,12 @@ void InitNet(Handle<Object> target) {
   NODE_SET_METHOD(target, "socketError", SocketError);
   NODE_SET_METHOD(target, "toRead", ToRead);
   NODE_SET_METHOD(target, "setNoDelay", SetNoDelay);
+  NODE_SET_METHOD(target, "setBroadcast", SetBroadcast);
+  NODE_SET_METHOD(target, "setTTL", SetTTL);
   NODE_SET_METHOD(target, "setKeepAlive", SetKeepAlive);
   NODE_SET_METHOD(target, "getsockname", GetSockName);
   NODE_SET_METHOD(target, "getpeername", GetPeerName);
+  NODE_SET_METHOD(target, "getaddrinfo", GetAddrInfo);
   NODE_SET_METHOD(target, "isIP", IsIP);
   NODE_SET_METHOD(target, "errnoException", CreateErrnoException);
 
@@ -927,8 +1312,11 @@ void InitNet(Handle<Object> target) {
   errno_symbol          = NODE_PSYMBOL("errno");
   syscall_symbol        = NODE_PSYMBOL("syscall");
   fd_symbol             = NODE_PSYMBOL("fd");
+  size_symbol           = NODE_PSYMBOL("size");
   address_symbol        = NODE_PSYMBOL("address");
   port_symbol           = NODE_PSYMBOL("port");
 }
 
 }  // namespace node
+
+NODE_MODULE(node_net, node::InitNet);
