@@ -1,4 +1,5 @@
-require("../common");
+common = require("../common");
+assert = common.assert
 
 var tcp = require("tcp"),
     sys = require("sys"),
@@ -10,25 +11,25 @@ var eofCount = 0;
 var server = tcp.createServer(function(socket) {
   socket.end();
 });
-server.listen(PORT);
+server.listen(common.PORT);
 
-var client = http.createClient(PORT);
+var client = http.createClient(common.PORT);
 
 client.addListener("error", function() {
-  sys.puts("ERROR!");
+  console.log("ERROR!");
   errorCount++;
 });
 
 client.addListener("end", function() {
-  sys.puts("EOF!");
+  console.log("EOF!");
   eofCount++;
 });
 
 var request = client.request("GET", "/", {"host": "localhost"});
-request.addListener('response', function(response) {
-  sys.puts("STATUS: " + response.statusCode);
-});
 request.end();
+request.addListener('response', function(response) {
+  console.log("STATUS: " + response.statusCode);
+});
 
 setTimeout(function () {
   server.close();
