@@ -1,4 +1,5 @@
-require("../common");
+common = require("../common");
+assert = common.assert
 
 var a = require('assert');
 
@@ -154,3 +155,6 @@ try {
 }
 assert.equal(true,threw,'a.doesNotThrow is not catching type matching errors');
 
+assert.throws(function () {assert.ifError(new Error('test error'))});
+assert.doesNotThrow(function(){assert.ifError(null)});
+assert.doesNotThrow(function(){assert.ifError()});
