@@ -1,8 +1,9 @@
-require('../common');
+common = require("../common");
+assert = common.assert
 path = require('path'),
 Buffer = require('buffer').Buffer
 fs = require('fs')
-fn = path.join(fixturesDir, 'write.txt');
+fn = path.join(common.fixturesDir, 'write.txt');
 
 
 foo = 'foo'
@@ -10,7 +11,8 @@ var fd = fs.openSync(fn, 'w');
 fs.writeSync(fd, foo);
 
 bar = 'bár'
-fs.writeSync(fd, new Buffer(bar), 0, Buffer.byteLength(bar));
+written = fs.writeSync(fd, new Buffer(bar), 0, Buffer.byteLength(bar));
+assert.ok(written > 3);
 fs.closeSync(fd);
 
 assert.equal(fs.readFileSync(fn), 'foobár');
