@@ -1,4 +1,5 @@
-require("../common");
+common = require("../common");
+assert = common.assert
 http = require("http");
 url = require("url");
 
@@ -10,17 +11,17 @@ var server = http.createServer(function (req, res) {
   res.writeHead(200, { "Content-Type": "text/plain"
                       , "Content-Length": body.length
                       });
-  res.write(body);
-  res.close();
+  res.end(body);
 });
-server.listen(PORT);
+server.listen(common.PORT);
 
-var client = http.createClient(PORT);
+var client = http.createClient(common.PORT);
 
 var body1 = "";
 var body2 = "";
 
 var req1 = client.request("/1")
+req1.end();
 req1.addListener('response', function (res1) {
   res1.setBodyEncoding("utf8");
 
@@ -30,15 +31,14 @@ req1.addListener('response', function (res1) {
 
   res1.addListener('end', function () {
     var req2 = client.request("/2");
+    req2.end();
     req2.addListener('response', function (res2) {
       res2.setBodyEncoding("utf8");
       res2.addListener('data', function (chunk) { body2 += chunk; });
       res2.addListener('end', function () { server.close(); });
     });
-    req2.close();
   });
 });
-req1.close();
 
 process.addListener("exit", function () {
   assert.equal(body1_s, body1);
