@@ -1,4 +1,5 @@
-require("../common");
+common = require("../common");
+assert = common.assert
 net = require("net");
 http = require("http");
 url = require("url");
@@ -14,17 +15,17 @@ try {
   have_openssl=true;
 } catch (e) {
   have_openssl=false;
-  puts("Not compiled with OPENSSL support.");
+  console.log("Not compiled with OPENSSL support.");
   process.exit();
-} 
+}
 
 var request_number = 0;
 var requests_sent = 0;
 var server_response = "";
 var client_got_eof = false;
-var caPem = fs.readFileSync(fixturesDir+"/test_ca.pem", 'ascii');
-var certPem = fs.readFileSync(fixturesDir+"/test_cert.pem", 'ascii');
-var keyPem = fs.readFileSync(fixturesDir+"/test_key.pem", 'ascii');
+var caPem = fs.readFileSync(common.fixturesDir+"/test_ca.pem", 'ascii');
+var certPem = fs.readFileSync(common.fixturesDir+"/test_cert.pem", 'ascii');
+var keyPem = fs.readFileSync(common.fixturesDir+"/test_key.pem", 'ascii');
 
 var credentials = crypto.createCredentials({key:keyPem, cert:certPem, ca:caPem});
 
@@ -61,7 +62,7 @@ var https_server = http.createServer(function (req, res) {
   if (req.id == 3) {
     assert.equal("bar", req.headers['x-x']);
     this.close();
-    //puts("server closed");
+    //console.log("server closed");
   }
   setTimeout(function () {
     res.writeHead(200, {"Content-Type": "text/plain"});
@@ -71,9 +72,9 @@ var https_server = http.createServer(function (req, res) {
 
 });
 https_server.setSecure(credentials);
-https_server.listen(PORT);
+https_server.listen(common.PORT);
 
-var c = net.createConnection(PORT);
+var c = net.createConnection(common.PORT);
 
 c.setEncoding("utf8");
 
