@@ -1,20 +1,21 @@
-require("../common");
-tcp = require("tcp");
+common = require("../common");
+assert = common.assert
+net = require("net");
 http = require("http");
 
 // wget sends an HTTP/1.0 request with Connection: Keep-Alive
 //
-// Sending back a chunked response to an HTTP/1.0 client would be wrong, 
-// so what has to happen in this case is that the connection is closed 
-// by the server after the entity body if the Content-Length was not 
+// Sending back a chunked response to an HTTP/1.0 client would be wrong,
+// so what has to happen in this case is that the connection is closed
+// by the server after the entity body if the Content-Length was not
 // sent.
 //
-// If the Content-Length was sent, we can probably safely honor the 
-// keep-alive request, even though HTTP 1.0 doesn't say that the 
-// connection can be kept open.  Presumably any client sending this 
-// header knows that it is extending HTTP/1.0 and can handle the 
-// response.  We don't test that here however, just that if the 
-// content-length is not provided, that the connection is in fact 
+// If the Content-Length was sent, we can probably safely honor the
+// keep-alive request, even though HTTP 1.0 doesn't say that the
+// connection can be kept open.  Presumably any client sending this
+// header knows that it is extending HTTP/1.0 and can handle the
+// response.  We don't test that here however, just that if the
+// content-length is not provided, that the connection is in fact
 // closed.
 
 var server_response = "";
@@ -25,11 +26,11 @@ var server = http.createServer(function (req, res) {
   res.writeHead(200, {"Content-Type": "text/plain"});
   res.write("hello ");
   res.write("world\n");
-  res.close();
+  res.end();
 })
-server.listen(PORT);
+server.listen(common.PORT);
 
-var c = tcp.createConnection(PORT);
+var c = net.createConnection(common.PORT);
 
 c.setEncoding("utf8");
 
@@ -39,19 +40,19 @@ c.addListener("connect", function () {
 });
 
 c.addListener("data", function (chunk) {
-  puts(chunk);
+  console.log(chunk);
   server_response += chunk;
 });
 
 c.addListener("end", function () {
   client_got_eof = true;
-  puts('got end');
-  c.close();
+  console.log('got end');
+  c.end();
 });
 
 c.addListener("close", function () {
   connection_was_closed = true;
-  puts('got close');
+  console.log('got close');
   server.close();
 });
 
