@@ -1,7 +1,8 @@
-require("../common");
+common = require("../common");
+assert = common.assert
 var path = require('path');
 
-debug("load test-module-loading.js");
+common.debug("load test-module-loading.js");
 
 var a = require("../fixtures/a");
 var c = require("../fixtures/b/c");
@@ -37,17 +38,24 @@ assert.equal("D", d4.D());
 
 assert.ok((new a.SomeClass) instanceof c.SomeClass);
 
-debug("test index.js modules ids and relative loading")
+common.debug("test index.js modules ids and relative loading")
 var one = require("../fixtures/nested-index/one"),
   two = require("../fixtures/nested-index/two");
 assert.notEqual(one.hello, two.hello);
 
-debug("test cycles containing a .. path");
+common.debug("test cycles containing a .. path");
 var root = require("../fixtures/cycles/root"),
   foo = require("../fixtures/cycles/folder/foo");
 assert.equal(root.foo, foo);
 assert.equal(root.sayHello(), root.hello);
 
+common.debug("test name clashes");
+// this one exists and should import the local module
+var my_path = require("./path");
+assert.equal(true, my_path.path_func instanceof Function);
+// this one does not exist and should throw
+assert.throws(function() { require("./utils")});
+
 var errorThrown = false;
 try {
   require("../fixtures/throws_error");
@@ -56,14 +64,24 @@ try {
   assert.equal("blah", e.message);
 }
 
-assert.equal(require('path').dirname(__filename), __dirname);
-
-require.async('../fixtures/a', function (err, a) {
-  if (err) throw err;
-  assert.equal("A", a.A());
+var errorThrownAsync = false;
+require.async("../fixtures/throws_error1", function(err, a) {
+  if (err) {
+    errorThrownAsync = true;
+    assert.equal("blah", err.message);
+  }
 });
 
-debug('load custom file types with registerExtension');
+assert.equal(require('path').dirname(__filename), __dirname);
+
+var asyncRun = false;
+require.async('../fixtures/a1', function (err, a) {
+  if (err) throw err;
+  assert.equal("A", a.A());
+  asyncRun = true;
+});
+
+common.debug('load custom file types with registerExtension');
 require.registerExtension('.test', function(content) {
   assert.equal("this is custom source\n", content);
 
@@ -72,7 +90,7 @@ require.registerExtension('.test', function(content) {
 
 assert.equal(require('../fixtures/registerExt').test, "passed");
 
-debug('load custom file types that return non-strings');
+common.debug('load custom file types that return non-strings');
 require.registerExtension('.test', function(content) {
   return {
     custom: 'passed'
@@ -80,6 +98,13 @@ require.registerExtension('.test', function(content) {
 });
 
 assert.equal(require('../fixtures/registerExt2').custom, 'passed');
+debug("load modules by absolute id, then change require.paths, and load another module with the same absolute id.");
+// this will throw if it fails.
+var foo = require("../fixtures/require-path/p1/foo");
+process.assert(foo.bar.expect === foo.bar.actual);
+
+assert.equal(require('../fixtures/foo').foo, 'ok',
+  'require module with no extension');
 
 process.addListener("exit", function () {
   assert.equal(true, a.A instanceof Function);
@@ -99,5 +124,9 @@ process.addListener("exit", function () {
 
   assert.equal(true, errorThrown);
 
-  puts("exit");
+  assert.equal(true, asyncRun);
+
+  assert.equal(true, errorThrownAsync);
+
+  console.log("exit");
 });
